
#    LUCI is an Universal Configuration Interface
#    Copyright (C) 1999-2001 Jrme SAUTRET <Jerome.Sautret@netcourrier.com>
#                                                                
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or   
#    (at your option) any later version.                              
#                                      
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of 
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
#    GNU General Public License for more details.                 
#                                               
#    You should have received a copy of the GNU General Public License
#    along with this program; see the file COPYING.  If not, write to 
#    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
#    Boston, MA 02111-1307, USA.

# import PyGTK module
from gtk import *
# import Python modules
import types, string, file
#import LUCI modules
import desc, file

debug=0

tooltips= GtkTooltips()
tooltips.set_delay(500)

entry_vsize=20

def create_tool_tip(dict_entry, entry):
    if dict_entry.has_key('description'):
	tooltips.set_tip(entry, dict_entry['description'], '')
	tooltips.enable()

def create_label(vbox, dict_entry):
    text=dict_entry['name'] + " : "
    if dict_entry.has_key('presence') and dict_entry['presence']=='optional':
        label=GtkCheckButton(text)
        create_tool_tip(dict_entry, label)
    else:
        label = GtkLabel(text)
    al=GtkAlignment(1, 0.5, 0, 0)
    al.set_usize(-1,entry_vsize)
    al.add(label)
    vbox.pack_start(al, expand=FALSE)
    label.show()
    al.show()

    return label

def set_widget_parameters(widget, dict_entry):
    if dict_entry.has_key('size'):
	widget.set_usize(dict_entry['size'],-1)    


def set_entry_parameters(entry, dict_entry):
    set_widget_parameters(entry, dict_entry)
    if dict_entry.has_key('value'):
	entry.set_text(dict_entry['value'])
    if dict_entry.has_key('max_length'):
	entry.set_max_length(dict_entry['max_length'])    

def create_entry(vbox, entry, dict_entry):
    al=GtkAlignment(0, 0.5, 0, 0)
    al.set_usize(-1,entry_vsize)
    al.add(entry)
    vbox.pack_start(al, expand=FALSE)
    create_tool_tip(dict_entry, entry)
    entry.show()
    al.show()    

def create_entry_string(vbox, dict_entry):
    entry = GtkEntry()
    dict_entry['_widget']=entry
    set_entry_parameters(entry, dict_entry)
    create_entry(vbox, entry, dict_entry)
    return entry

def create_entry_filename(vbox, dict_entry):
    entry = GtkEntry()
    dict_entry['_widget']=entry
    set_entry_parameters(entry, dict_entry)
    create_tool_tip(dict_entry, entry)    
    entry.show()

    button=GtkButton("Browse...")

    def browse(button, entry=entry):
	button.set_sensitive(FALSE)
	file_box=GtkFileSelection("file selection")

	def file_box_cancel(widget, button=button):
	    button.set_sensitive(TRUE)
	
	def file_box_ok(widget, entry=entry, file_box=file_box):
	    entry.set_text(file_box.get_filename())
	    file_box.destroy()
	
	file_box.set_filename(entry.get_text())
	file_box.connect("destroy", file_box_cancel)
	file_box.ok_button.connect("clicked", file_box_ok)
	file_box.cancel_button.connect("clicked", file_box.destroy)
	file_box.show()
    
    button.connect("clicked", browse)
    button.show()
    
    hbox = GtkHBox(spacing=2)
    hbox.pack_start(entry)
    hbox.pack_start(button)
    create_entry(vbox, hbox, dict_entry)
    return hbox

def create_entry_integer(vbox, dict_entry):
    min=int(dict_entry['integer_limits'][0])
    max=int(dict_entry['integer_limits'][1])
    if not dict_entry.has_key('value'):
        dict_entry['value']=`min`
    value=string.atoi(dict_entry['value'])
    spin=GtkSpinButton(GtkAdjustment(value,min,max,1,((max - min)/10)+1),
                       0.5, 0)
    dict_entry['_widget']=spin
    set_entry_parameters(spin, dict_entry)
    create_entry(vbox, spin, dict_entry)
    return spin

def create_entry_list(vbox, dict_entry):
    combo = GtkCombo()
    list2=[]
    for i in dict_entry['list_values']:
	if type(i)==types.TupleType:
	    list2.append(i[0])
	else:
	    list2.append(i)
    combo.set_popdown_strings(list2)
    dict_entry['_widget']=combo.entry
    set_entry_parameters(combo.entry, dict_entry)
    create_tool_tip(dict_entry, combo.entry)
    create_entry(vbox, combo, dict_entry)
    return combo

def add_entry(left_box, right_box, dict_entry):
    "Add the entry in the container."

    try:
        label=create_label(left_box, dict_entry)
        
        if not dict_entry.has_key('value_type'):
            dict_entry['value_type']='string'
        value_type=dict_entry['value_type']
        entry_type=type(value_type)
        if value_type=='string':
            entry=create_entry_string(right_box, dict_entry)
        elif value_type=='integer':
            entry=create_entry_integer(right_box, dict_entry)
        elif value_type=='list':
            entry=create_entry_list(right_box, dict_entry)
        elif value_type=='filename':
            entry=create_entry_filename(right_box, dict_entry)
        else:
            print "Error, unknown value type '"+value_type+"' for entry '"+\
                  dict_entry['name']+"'."
            raise SystemExit, 8
            
        if not dict_entry.has_key('value'):
            dict_entry['_active']=0
                
        if (dict_entry.has_key('presence') and \
            dict_entry['presence']=='optional'):
            def check_button_clicked(button, entry=entry,
                                     dict_entry=dict_entry):
                active=button.get_active()
                entry.set_sensitive(active)
                dict_entry['_active']=active
                
            label.connect("clicked", check_button_clicked)
            label.set_active(dict_entry['_active'])
            check_button_clicked(label)

    except KeyError, k:
        print "Error, parameter '"+k.args[0]+"' is missing for entry '"+\
              dict_entry['_entry_name']+"'."
        raise SystemExit, 10


def get_widget_value(entry):
    widget_type=entry['_widget'].get_name()
    value=entry['_widget'].get_text()
    if entry['value_type']=='list':
	for i in entry['list_values']:
	    if type(i)==types.TupleType:
		if i[0]==value:
		    value=i[1]
		    break
    if type(value)!=types.StringType:
	value=`value`
    return value

def get_entries_values(widget, win, config, entries):
    for entry in entries.keys():
        if debug:
            print entries[entry]['name']
            print get_widget_value(entries[entry])
	entries[entry]['value']=get_widget_value(entries[entry])
    file.write_file(config, entries)
    if config.has_key('tree'):
        win.destroy()
    else:
        mainquit()
        raise SystemExit, 0 

def ask_new_file(win, config, entries):
    """
    ask confirmation for creating a new file
    config: dict of global parameters
    entries: dict of entries parameters
    """
    
    dialog = GtkDialog()
    label = GtkLabel("\n    File "+config['config_file_name']+
                     " doesn't exist. Create it ?    \n")
    ok_button = GtkButton("Okay")
    notok_button = GtkButton("Forget it")

    def destroy_dialog(button, dialog=dialog, config=config):
        dialog.destroy()
        if not config.has_key('tree'):
            quit_cancel(None)

    def OK_cliked(button, entries, dialog=dialog, win=win, config=config):
        dialog.destroy()
        entries_window(win, config, entries)
    
    ok_button.connect("clicked", OK_cliked, entries)
    dialog.action_area.add(ok_button)
    notok_button.connect("clicked", destroy_dialog)
    dialog.action_area.add(notok_button)
    
    dialog.vbox.add(label)
    dialog.set_modal(TRUE)
    dialog.set_position(WIN_POS_MOUSE)
    dialog.show_all()

def init_entries(win, config):
    "read desc file & config file for initialisation of config & entries"
    module=desc.get_module(config['desc_file_name'])
    config_tmp, entries=desc.get_entries(module)
    config.update(config_tmp)
    file.init_params(config)
    if file.read_file(config, entries)=="NoConfigFile" \
       and not config.has_key('param_force'):
        ask_new_file(win, config, entries)
    else:
        entries_window(win, config, entries)

def quit_cancel(button):
    "quit when cancel is pressed"
    mainquit()
    raise SystemExit, 1
        
def entries_window(win, config, entries):
    "Create the window containing the entries"

    scroll=GtkScrolledWindow()
    scroll.set_policy(POLICY_AUTOMATIC,POLICY_AUTOMATIC)

    main_vbox = GtkVBox(spacing=2)

    hbox = GtkHBox(spacing=2)
    left_vbox = GtkVBox(spacing=5)
    right_vbox = GtkVBox(spacing=5)
    hbox.pack_start(left_vbox, expand=FALSE)
    hbox.pack_start(right_vbox, expand=FALSE)
    
    if config.has_key('entries'):
	list_entries=config['entries']
    else:
	list_entries=entries.keys()
    for entry in list_entries:
	add_entry(left_vbox, right_vbox, entries[entry])

    buttons_hbox = GtkHButtonBox()
    buttons_hbox.set_layout_default(BUTTONBOX_SPREAD)
    
    button_OK=GtkButton('OK')
    button_OK.connect("clicked", get_entries_values, scroll, config, entries)
    buttons_hbox.pack_start(button_OK)

    button_cancel=GtkButton('Cancel')

    if config.has_key('tree'):
        button_cancel.connect("clicked", scroll.destroy)
    else:
        button_cancel.connect("clicked", quit_cancel)
    buttons_hbox.pack_start(button_cancel)

    def about_window(button, config_file=config['config_file_name'],
                     desc_file=config['desc_file_name'],
                     version=config['luci_version']):
        button.set_sensitive(FALSE)
	win=GtkWindow(type=WINDOW_DIALOG, title="About LUCI")
        win.set_border_width(10)

        box=GtkVBox(spacing=20)
        label=GtkLabel("LUCI is an Universal Configuration Interface")
        box.pack_start(label)
        label.show()
        label=GtkLabel("Version "+version)
        box.pack_start(label)
        label.show()
        label=GtkLabel(
            "Copyright  1999-2001 Jrme SAUTRET <Jerome.Sautret@netcourrier.com>")
        box.pack_start(label)
        label.show()
        label=GtkLabel(
            "Licenced under GNU General Public Licence (GPL)."+
            " See COPYING file for more details or type 'luci --license'.")
        label.set_line_wrap(TRUE)
        box.pack_start(label)
        label.show()
        label=GtkLabel("Homepage: http://LUCI.zubro.cjb.net")
        box.pack_start(label)
        label.show()
        label=GtkLabel('You are currently editing configuration file "' +
                       config_file + '" using desciption file "'+
                       desc_file+'".')
        label.set_line_wrap(TRUE)
        box.pack_start(label)
        label.show()
        win.add(box)
        box.show()

        close_button=GtkButton('Close')
        box.pack_start(close_button)
        
	def about_destroy(widget, button=button, win=win):
            button.set_sensitive(TRUE)
	
	win.connect("destroy", about_destroy)
	close_button.connect("clicked", win.destroy)
        close_button.show()
	win.show()

    button_about=GtkButton('About')
    button_about.connect("clicked", about_window)
    buttons_hbox.pack_start(button_about)

    button_OK.show()
    button_cancel.show()
    button_about.show()


    if (config.has_key('title')):
        if config['title']:
            frame=GtkFrame(config['title'])
        else:
            frame=GtkFrame()
        frame.set_border_width(5)
        hbox.set_border_width(10)
        frame.add(hbox)
        main_vbox.pack_start(frame, expand=FALSE)
        frame.show()
    else:
        main_vbox.pack_start(hbox, expand=FALSE)
        
    main_vbox.pack_start(buttons_hbox, expand=FALSE)

    left_vbox.show()
    right_vbox.show()
    hbox.show()
    buttons_hbox.show()
    main_vbox.show()
    size=main_vbox.size_request()
    scroll.set_usize(size[0]+15, size[1]+15)

    scroll.add_with_viewport(main_vbox)
    win.add(scroll)
    scroll.show()
    #win.set_resize_mode(RESIZE_QUEUE)
    #win.get_toplevel().queue_resize()
    #win.get_toplevel().resize_children()
    win.show()
